@@ -113,7 +113,7 @@ def get_pdf_help(missing_libs=""):
         pip install 'pycairo>=1.18' 'weasyprint<=0.42.3'
 
 
-    3. Finally, install the full Noto Font package:
+    3. Finally, install the full Noto Font and Roboto Mono packages:
 
        * Download the full font file from:
          https://noto-website-2.storage.googleapis.com/pkgs/Noto-unhinted.zip
@@ -122,6 +122,10 @@ def get_pdf_help(missing_libs=""):
 
        * Follow the installation instructions at
          https://www.google.com/get/noto/help/install/
+
+       * Go to https://fonts.google.com/specimen/Roboto+Mono, and download the
+         font.  Follow the installation instructions above, as applied to this
+         download.
     
     With these libraries, modules, and fonts installed and available to
     xml2rfc, the --pdf switch will be enabled.
@@ -179,6 +183,8 @@ def main():
                            help='convert vocabulary version 2 XML to version 3')
     formatgroup.add_option('', '--preptool', action='store_true',
                            help='run preptool on the input')
+    formatgroup.add_option('', '--unprep', action='store_true',
+                           help='reduce prepped xml to unprepped')
     formatgroup.add_option('', '--info', action='store_true',
                            help='generate a JSON file with anchor to section lookup information')
     optionparser.add_option_group(formatgroup)
@@ -390,7 +396,7 @@ def main():
             options.output_path = options.basename
             options.basename = None
     #
-    num_formats = len([ o for o in [options.raw, options.text, options.nroff, options.html, options.expand, options.v2v3, options.preptool, options.info, options.pdf ] if o])
+    num_formats = len([ o for o in [options.raw, options.text, options.nroff, options.html, options.expand, options.v2v3, options.preptool, options.info, options.pdf, options.unprep ] if o])
     if num_formats > 1 and (options.filename or options.output_filename):
         sys.exit('Cannot use an explicit output filename when generating more than one format, '
                  'use --path instead.')
@@ -625,6 +631,16 @@ def main():
             preptool.write(filename)
             options.output_filename = None
 
+        if options.unprep:
+            xmlrfc = parser.parse(remove_comments=False, quiet=True, add_xmlns=True)
+            filename = options.output_filename
+            if not filename:
+                filename = basename.replace('.prepped','') + '.plain.xml'
+                options.output_filename = filename
+            unprep = xml2rfc.UnPrepWriter(xmlrfc, options=options, date=options.date)
+            unprep.write(filename)
+            options.output_filename = None
+
         if options.text and not options.legacy:
             xmlrfc = parser.parse(remove_comments=False, quiet=True, add_xmlns=True)
             filename = options.output_filename
