@@ -111,6 +111,7 @@ class TextWrapper(textwrap.TextWrapper):
                                     drop_whitespace=drop_whitespace) 
                 else:
                     wrapped.append(part)
+                initial = ''
             return wrapped
 
         # Set indentation
@@ -176,7 +177,7 @@ class TextWrapper(textwrap.TextWrapper):
 
     def fill(self, *args, **kwargs):
         kwargs.pop('fill', None)
-        return "\n".join(self.wrap(*args, **kwargs))
+        return "\u2028".join(self.wrap(*args, **kwargs))
 
 
 class TextSplitter(textwrap.TextWrapper):
@@ -653,3 +654,9 @@ def is_htmlblock(h):
         'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'li', 'nav', 'ol', 'p', 'pre', 'script', 'section',
         'table', 'ul', ])
 
+def slugify(s):
+    s = s.strip().lower()
+    s = re.sub(r'[^\w\s/|@=-]', '', s)
+    s = re.sub(r'[-_\s/|@=]+', '_', s)
+    s = s.strip('_')
+    return s
